@@ -3,7 +3,7 @@
 #----------------------------------------------------------------------
 # xoutil._meta3
 #----------------------------------------------------------------------
-# Copyright (c) 2013 Merchise Autrement and Contributors
+# Copyright (c) 2013, 2014 Merchise Autrement and Contributors
 # All rights reserved.
 #
 # This is free software; you can redistribute it and/or modify it under
@@ -27,7 +27,7 @@ __author__ = "Manuel Vázquez Acosta <mva.led@gmail.com>"
 __date__ = "Mon Apr 29 15:34:11 2013"
 
 
-def metaclass(meta, collect=True):
+def metaclass(meta):
     class base:
         pass
 
@@ -44,7 +44,9 @@ def metaclass(meta, collect=True):
         def __init__(self, name, bases, attrs):
             pass
 
-    class __inner__(base, metaclass=inner_meta):
-        pass
+    # Hide this stuff from Python 2.7 compiler so several static analysis tools
+    # (like pylint and other) don't complain about a SyntaxError.
+    exec(compile('class __inner__(base, metaclass=inner_meta):pass',
+                 __file__, 'exec'))
 
-    return __inner__
+    return locals()['__inner__']
