@@ -3,7 +3,7 @@
 #----------------------------------------------------------------------
 # xoutil.objects
 #----------------------------------------------------------------------
-# Copyright (c) 2013 Merchise Autrement and Contributors
+# Copyright (c) 2013, 2014 Merchise Autrement and Contributors
 # Copyright (c) 2012 Medardo Rodríguez
 # All rights reserved.
 #
@@ -28,37 +28,28 @@ from xoutil.compat import py3k as _py3k
 from xoutil.deprecation import deprecated
 
 from xoutil.names import strlist as slist
-__all__ = slist('nameof', 'smart_getter', 'smart_getter_and_deleter',
+__all__ = slist('smart_getter', 'smart_getter_and_deleter',
                 'xdir', 'fdir', 'validate_attrs', 'get_first_of',
                 'get_and_del_first_of', 'smart_getattr', 'get_and_del_attr',
-                'setdefaultattr', 'full_nameof', 'copy_class', 'smart_copy',
+                'setdefaultattr', 'copy_class', 'smart_copy',
                 'extract_attrs')
 del slist
 
 __docstring_format__ = 'rst'
 __author__ = 'manu'
 
+
+_INVALID_CLASS_TYPE_MSG = '``cls`` must be a class not an instance'
+
+# === Helper functions ====
+
+_len = lambda x: len(x) if x else 0
+
 # These two functions can be use to always return True or False
 _true = lambda *args, **kwargs: True
 _false = lambda *args, **kwargs: False
 
 
-# TODO: Deprecate and restructure all its uses
-@deprecated('xoutil.names.nameof')
-def nameof(target):
-    '''Gets the name of an object.
-
-    .. deprecated:: 1.4.0 Use :func:`xoutil.names.nameof`.
-
-    '''
-    from xoutil.names import nameof as wrapped
-    from xoutil.compat import str_base
-    if isinstance(target, str_base):
-        return wrapped(target, depth=2, inner=True)
-    else:
-        return wrapped(target, depth=2, inner=True, typed=True)
-
-
 def smart_getter(obj):
     '''Returns a smart getter for `obj`.
 
@@ -74,8 +65,6 @@ def smart_getter(obj):
     else:
         return lambda attr, default=None: getattr(obj, attr, default)
 
-smart_get = deprecated(smart_getter)(smart_getter)
-
 
 def smart_getter_and_deleter(obj):
     '''Returns a function that get and deletes either a key or an attribute of
@@ -94,8 +83,6 @@ def smart_getter_and_deleter(obj):
     else:
         return partial(get_and_del_attr, obj)
 
-smart_get_and_del = deprecated(smart_getter_and_deleter)(smart_getter_and_deleter)
-
 
 def is_private_name(name):
     '''Return if `name` is private or not.'''
@@ -149,6 +136,136 @@ def attrclass(obj, name):
         return next((cls for cls in cls_chcks if cls is not None), None)
 
 
+# TODO: [med] [manu] Decide if it's best to create a 'xoutil.inspect' that
+# extends the standard library module 'inspect' and place this
+# signature-dealing functions there. Probably, to be consistent, this imposes a
+# refactoring of some of 'xoutil.types' and move all the "is_classmethod",
+# "is_staticmethod" and inspection-related functions there.
+def get_method_function(cls, method_name):
+    '''Get definition function given in its :param:`method_name`.
+
+    There is a difference between the result of this function and
+    ``getattr(cls, method_name)`` because the last one return the unbound
+    method and this a python function.
+
+    '''
+    if not isinstance(cls, type):
+        cls = cls.__class__
+    mro = cls.mro()
+    i, res = 0, None
+    while not res and (i < len(mro)):
+        sc = mro[i]
+        method = sc.__dict__.get(method_name)
+        if callable(method):
+            res = method
+        else:
+            i += 1
+    return res
+
+
+def build_documentation(cls, get_doc=None, deep=1):
+    '''Build a proper documentation from a class :param:`cls`.
+
+    Classes are recursed in MRO until process all levels (:param:`deep`)
+    building the resulting documentation.
+
+    The function :param:`get_doc` get the documentation of a given class. If
+    no function is given, then attribute ``__doc__`` is used.
+
+    '''
+    from xoutil.string import safe_decode
+    assert isinstance(cls, type), _INVALID_CLASS_TYPE_MSG
+    if deep < 1:
+        deep = 1
+    get_doc = get_doc or (lambda c: c.__doc__)
+    mro = cls.mro()
+    i, level, used, res = 0, 0, {}, ''
+    while (level < deep) and (i < len(mro)):
+        sc = mro[i]
+        doc = get_doc(sc)
+        if doc:
+            doc = safe_decode(doc).strip()
+            key = sc.__name__
+            docs = used.setdefault(key, set())
+            if doc not in docs:
+                docs.add(doc)
+                if res:
+                    res += '\n\n'
+                res += '=== <%s> ===\n\n%s' % (key, doc)
+                level += 1
+        i += 1
+    return res
+
+
+def fix_class_documentation(cls, ignore=None, min_length=10, deep=1,
+                            default=None):
+    '''Fix the documentation for the given class using its super-classes.
+
+    This function may be useful for shells or Python Command Line Interfaces
+    (CLI).
+
+    If :param:`cls` has an invalid documentation, super-classes are recursed
+    in MRO until a documentation definition was made at any level.
+
+    :param:`ignore` could be used to specify which classes to ignore by
+                    specifying its name in this list.
+
+    :param:`min_length` specify that documentations with less that a number of
+                        characters, also are ignored.
+
+    '''
+    assert isinstance(cls, type), _INVALID_CLASS_TYPE_MSG
+    if _len(cls.__doc__) < min_length:
+        ignore = ignore or ()
+        def get_doc(c):
+            if (c.__name__ not in ignore) and _len(c.__doc__) >= min_length:
+                return c.__doc__
+            else:
+                return None
+        doc = build_documentation(cls, get_doc, deep)
+        if doc:
+            cls.__doc__ = doc
+        elif default:
+            cls.__doc__ = default(cls) if callable(default) else default
+
+
+def fix_method_documentation(cls, method_name, ignore=None, min_length=10,
+                             deep=1, default=None):
+    '''Fix the documentation for the given class using its super-classes.
+
+    This function may be useful for shells or Python Command Line Interfaces
+    (CLI).
+
+    If :param:`cls` has an invalid documentation, super-classes are recursed
+    in MRO until a documentation definition was made at any level.
+
+    :param:`ignore` could be used to specify which classes to ignore by
+                    specifying its name in this list.
+
+    :param:`min_length` specify that documentations with less that a number of
+                        characters, also are ignored.
+
+    '''
+    assert isinstance(cls, type), _INVALID_CLASS_TYPE_MSG
+    method = get_method_function(cls, method_name)
+    if method and _len(method.__doc__) < min_length:
+        ignore = ignore or ()
+        def get_doc(c):
+            if (c.__name__ not in ignore):
+                method = c.__dict__.get(method_name)
+                if callable(method) and _len(method.__doc__) >= min_length:
+                    return method.__doc__
+                else:
+                    return None
+            else:
+                return None
+        doc = build_documentation(cls, get_doc, deep)
+        if doc:
+            method.__doc__ = doc
+        elif default:
+            method.__doc__ = default(cls) if callable(default) else default
+
+
 # TODO: [med] Explain "valid" in documentation.
 def fulldir(obj):
     '''Return a set with all valid attribute names defined in `obj`'''
@@ -347,7 +464,6 @@ def get_first_of(source, *keys, **kwargs):
         True
 
     '''
-    from xoutil import Unset
     from xoutil.types import is_collection
     def inner(source):
         get = smart_getter(source)
@@ -401,7 +517,6 @@ def get_and_del_first_of(source, *keys, **kwargs):
         True
 
     '''
-    from xoutil import Unset
     from xoutil.types import is_collection
     def inner(source):
         get = smart_getter_and_deleter(source)
@@ -446,7 +561,6 @@ def smart_getattr(name, *sources, **kwargs):
     :func:`get_first_of`_.
 
     '''
-    from xoutil import Unset
     from xoutil.iterators import dict_update_new
     dict_update_new(kwargs, {'default': Unset})
     return get_first_of(sources, name, **kwargs)
@@ -471,7 +585,6 @@ def get_and_del_attr(obj, name, default=None):
         True
 
     '''
-    from xoutil import Unset
     res = getattr(obj, name, Unset)
     if res is Unset:
         res = default
@@ -499,7 +612,6 @@ def get_and_del_key(d, key, default=None):
         True
 
     '''
-    from xoutil import Unset
     res = d.get(key, Unset)
     if res is Unset:
         res = default
@@ -589,7 +701,6 @@ def setdefaultattr(obj, name, value):
         'a'
 
     '''
-    from xoutil import Unset
     res = getattr(obj, name, Unset)
     if res is Unset:
         if isinstance(value, lazy):
@@ -599,28 +710,6 @@ def setdefaultattr(obj, name, value):
     return res
 
 
-# TODO: Use "xoutil.names.nameof" with "full=True, inner=True, typed=True"
-@deprecated('xoutil.names.nameof')
-def full_nameof(target):
-    '''Gets the full name of an object:
-
-    .. deprecated:: 1.4.0 Use :func:`xoutil.names.nameof` with the `full`
-       argument.
-
-    '''
-    from xoutil.compat import py3k, str_base
-    if isinstance(target, str_base):
-        return target
-    else:
-        if not hasattr(target, '__name__'):
-            target = type(target)
-        res = target.__name__
-        mod = getattr(target, '__module__', '__')
-        if not mod.startswith('__') and (not py3k or mod != 'builtins'):
-            res = '.'.join((mod, res))
-        return res
-
-
 def copy_class(cls, meta=None, ignores=None, new_attrs=None):
     '''Copies a class definition to a new class.
 
@@ -750,7 +839,6 @@ def smart_copy(*args, **kwargs):
     '''
     from collections import Mapping, MutableMapping
     from xoutil.compat import callable, str_base
-    from xoutil import Unset
     from xoutil.types import FunctionType as function
     from xoutil.types import is_collection, Required
     from xoutil.types import DictProxyType
@@ -762,8 +850,8 @@ def smart_copy(*args, **kwargs):
                         % kwargs.keys()[0])
     if defaults is Unset and len(args) >= 3:
         args, last = args[:-1], args[-1]
-        if isinstance(last, bool) or isinstance(last, function):
-            defaults = last
+        if isinstance(last, bool) or isinstance(last, function) or last is None:
+            defaults = last if last is not None else False
             sources, target = args[:-1], args[-1]
         else:
             sources, target, defaults = args, last, False
@@ -869,7 +957,7 @@ metaclass.__doc__ = '''Defines the metaclass of a class using a py3k-looking
        >>> Spam.__bases__ == (dict, )
        True
 
-    .. note::
+    .. warning::
 
        You should always place your metaclass declaration *first* in the list
        of bases. Doing otherwise triggers *twice* the metaclass' constructors
@@ -877,7 +965,7 @@ metaclass.__doc__ = '''Defines the metaclass of a class using a py3k-looking
 
        If your metaclass has some non-idempotent side-effect (such as
        registration of classes), then this would lead to unwanted double
-       registration of the class.
+       registration of the class::
 
           >>> class BaseMeta(type):
           ...     classes = []
@@ -955,15 +1043,15 @@ def traverse(obj, path, default=Unset, sep='.', getter=None):
     the signature of `getattr`.
 
     '''
-    unset = object()
+    notfound = object()
     current = obj
     if not getter:
         getter = lambda o, a, default=None: smart_getter(o)(a, default)
     attrs = path.split(sep)
-    while current is not unset and attrs:
+    while current is not notfound and attrs:
         attr = attrs.pop(0)
-        current = getter(current, attr, unset)
-    if current is unset:
+        current = getter(current, attr, notfound)
+    if current is notfound:
         if default is Unset:
             raise AttributeError(attr)
         else:
